# -*- coding: utf-8 -*-

import Tkinter
import ttk
import types

import libmisc
from Icon import Icon
from ScrolledWidgets import ScrolledText
from UnicodeHandler import fsdecode, fsencode

#############################################################################

class BaseDialog(Tkinter.Toplevel):
    def __init__(self, master=None, buttons=(), icon='',
                            message='', detail='', **kw):

        Tkinter.Toplevel.__init__(self, master, **kw)
        self.withdraw()
        self.title('Update Notifier Applet')
        self.protocol('WM_DELETE_WINDOW', self.hide)
        self._hotkeys = []

        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(0, weight=1)

        self.body = Tkinter.Frame(self)
        self.body.grid(row=0, column=0, sticky='news')
        self.body.grid_rowconfigure(1, weight=1)
        self.body.grid_columnconfigure(0, weight=1)

        self.head = Tkinter.Frame(self.body)
        self.head.grid(row=0, column=0, sticky='ew')
        self.head.grid_columnconfigure(1, weight=1)
        self.head.grid_rowconfigure(0, weight=1)

        self.img = Icon(icon)
        self.icon = Tkinter.Label(self.head, image=self.img)
        self.icon.grid(row=0, column=0, rowspan=2, padx=10, pady=10, sticky='w')

        self.message = Tkinter.Label(self.head, font='TkCaptionFont',
                                    text=message, anchor='w')
        self.message.grid(row=0, column=1, padx=10, pady=(10, 3), sticky='w')
        self.detail = Tkinter.Label(self.head, justify='left', text=detail)
        self.detail.grid(row=1, column=1, padx=10, pady=(3, 10), sticky='w')

        self.frame = Tkinter.Frame(self.body)
        self.frame.grid(row=1, column=0, sticky='news')

        self._buttons = []
        if buttons:
            # buttonbox geometry borrowed from msgbox.tcl
            self.buttonbox = Tkinter.Frame(self)
            self.buttonbox.grid(row=1, column=0)
            #self.tk.call('grid', 'anchor', self.buttonbox._w, 'center')
            libmisc.grid_anchor(self.buttonbox, 'center')

            for s in buttons:
                button = ttk.Button(self.buttonbox, text=s)
                button.grid(row=0, column=buttons.index(s),
                            pady='2m', padx='3m', sticky='ew')
                self.buttonbox.grid_columnconfigure(
                            buttons.index(s), uniform=1)
                button.bind('<FocusIn>', self._default_on)
                button.bind('<FocusOut>', self._default_off)
                self._buttons.append(button)

    def _default_off(self, event):
        event.widget.configure(default='normal')

    def _default_on(self, event):
        event.widget.configure(default='active')

    def assign_hotkey(self, button):
        i = 0
        letters = 'abcdefghijklmnopqrstuvwxyz'
        for char in button['text']:
            # if we call fsencode for the text string as a whole, counting
            # might get out of sync if we have a two-byte character
            # like a german umlaut in the button text
            c = fsencode(char).lower()
            if len(c) == 1 and c in letters and ord(c) < 128 and not c in self._hotkeys:
                self._hotkeys.append(c)
                button.configure(underline=i)
                seq, cmd = '<Alt-%s>' % c, button['command']
                # button['command'] is a str, so no need to bother with events
                self.bind(seq, cmd)
                break
            i += 1
        return seq

    def assign_hotkeys(self):
        for b in self._buttons:
            self.assign_hotkey(b)

    def buttons(self, index=None):
        if index is None:
            return self._buttons
        return self._buttons[index]

    def centeronscreen(self):
        parent = self.winfo_parent()
        if type(parent) == types.StringType:
            parent = self._nametowidget(parent)

        # Find size of window.
        self.update_idletasks()
        width = self.winfo_width()
        height = self.winfo_height()
        if width == 1 and height == 1:
            # If the window has not yet been displayed, its size is
            # reported as 1x1, so use requested size.
            width = self.winfo_reqwidth()
            height = self.winfo_reqheight()

        # Place in centre of screen:
        x = (self.winfo_screenwidth() - width) / 2 - parent.winfo_vrootx()
        y = (self.winfo_screenheight() - height) / 2 - parent.winfo_vrooty()
        if x < 0:
            x = 0
        if y < 0:
            y = 0
        return '+%d+%d' % (x, y)

    def hide(self):
        self.withdraw()

    def show(self):
        if not self.winfo_ismapped():
            self.geometry(self.centeronscreen())
            self.deiconify()
            self.wait_visibility()
        self.lift()

#############################################################################

class Msgbox(BaseDialog):
    def __init__(self, parent=None, message='', detail='', text='',
                    icon='error', buttons=(_('OK'),), default=0, **kw):
        BaseDialog.__init__(self, parent, buttons=buttons, message=message,
                            detail=detail, icon='dialog_%s' % icon, **kw)
        self.parent = parent or Tkinter._default_root

        wl = self.winfo_screenwidth() / 4
        if text:
            self.text = ScrolledText(self.frame, width=80, height=12)
            self.text.pack(fill='both', expand=1, padx=10, pady=10)
            self.text.insert('end', text)
            self.text.configure(state='disabled')

            wl1 = self.text.winfo_reqwidth() - 120
            if wl1 > 150: # sanity check
                wl = wl1
        self.message.configure(wraplength=wl)
        self.detail.configure(wraplength=wl)

        self.result = None
        self.defaultbutton = None

        for b in self.buttons():
            callback = (self.register(self._buttonpressed), b['text'])
            b.configure(command=callback)
            self.assign_hotkey(b)

        try:
            self.defaultbutton = self.buttons()[default]
        except(IndexError):
            self.defaultbutton = self.buttons()[0]
            print('Error: default button #%i does not exist.' % default)

    def _buttonpressed(self, text):
        self.result = text
        self.hide()

    def hide(self, *event):
        self.destroy()

    def show(self):
        BaseDialog.show(self)
        self.defaultbutton.focus_set()
        self.grab_set()
        try:
            if self.parent and self.parent.winfo_viewable():
                self.transient(self.parent)
        except:
            pass
        self.wait_window()
        return self.result

#############################################################################

def msgbox_askokcancel(parent=None, message='', detail='', text='',
                        default=0):
    m = Msgbox(parent, message, detail, text, icon='question',
                        buttons=(_('OK'), _('Cancel')), default=default)
    return m.show() == _('OK')

def msgbox_askyesno(parent=None, message='', detail='', text='',
                        default=0):
    m = Msgbox(parent, message, detail, text, icon='question',
                        buttons=(_('Yes'), _('No')), default=default)
    return m.show() == _('Yes')

def msgbox_askquestion(parent=None, message='', detail='', text='',
                        buttons=('OK',), default=0):
    m = Msgbox(parent, message, detail, text, icon='question',
                        buttons=buttons, default=default)
    return m.show()

def msgbox_showinfo(parent=None, message='', detail='', text=''):
    m = Msgbox(parent, message, detail, text, icon='info')
    m.show()

def msgbox_showerror(parent=None, message='', detail='', text=''):
    m = Msgbox(parent, message, detail, text, icon='error')
    m.show()

