# -*- coding: utf-8 -*-
'''Spinbox widget with strict validation.'''

import Tkinter

class Counter(Tkinter.Frame):
    def __init__(self, master, label=None, rlabel=None, from_=0, to=100,
                 increment=1, value=0, datatype='integer', digits=0,
                 width=5, command=None, invcmd='bell', **kw):
        Tkinter.Frame.__init__(self, master, **kw)

        self._command = command
        self._datatype = datatype
        self._oldvalue = value
        if invcmd == 'bell':
            invcmd = self.bell

        self.variable = Tkinter.StringVar()
        self.label, self.rlabel = None, None
        if label:
            self.label = Tkinter.Label(self, text=label)
            self.label.pack(side='left', padx=(0, 5))
        self.spinbox = Tkinter.Spinbox(self, from_=from_, to=to,
                                       increment=increment, validate='key',
                                       textvariable=self.variable,
                                       invcmd=invcmd, width=width)
        self.spinbox.pack(side='left', fill='x', expand=1)
        if rlabel:
            self.rlabel = Tkinter.Label(self, text=rlabel)
            self.rlabel.pack(side='left', padx=(5, 0))
        if digits:
            format = '%' + '0.%df' % digits
            self.spinbox.configure(format=format)

        self.set(value)
        vcmd = (self.register(self._validate), '%s', '%P')
        self.spinbox.configure(vcmd=vcmd)
        self.spinbox.bind('<FocusOut>', self._on_focusout)

    def _on_focusout(self, event):
        # if the spinbox is set to an illegal value,
        # restore the last valid value when focus leaves the widget
        self.get()

    def _validate(self, old, new):
        if (old != '') and (float(self.spinbox['from']) <= \
                            float(old) <= float(self.spinbox['to'])):
            self._oldvalue = old
        if new == '':
            return 1
        try:
            if self._datatype == 'integer':
                x = int(new)
            elif self._datatype == 'float':
                x = float(new)
            if 0 <= x <= float(self.spinbox['to']):
                if self._command and (new != self._oldvalue):
                    self._command(x)
                return 1
            return 0
        except(ValueError):
            return 0

    def bind(self, sequence, callback, *args):
        '''Convenience method.'''
        return self.spinbox.bind(sequence, callback, *args)

    def focus_get(self):
        '''Convenience method.'''
        return self.spinbox.focus_get()

    def focus_set(self):
        '''Convenience method.'''
        self.spinbox.focus_set()
    focus = focus_set

    def get(self, raw=False):
        # if raw=True return the value of the variable without validation;
        # this is needed e.g. when tracing changes to the widget's value
        if raw:
            # FIXME: isn't this too simple?
            try:
                if self._datatype == 'integer':
                    return int(self.variable.get())
                else:
                    return float(self.variable.get())
            except(ValueError):
                return None
        else:
            val = self.spinbox.get()
            if val == '':
                self.set(self._oldvalue)
            # now see if the current value is smaller than the allowed minimum
            # this is allowed for editing but not for retrieving the value
            elif self._datatype == 'integer':
                if int(val) < float(self.spinbox['from']):
                    self.set(self._oldvalue)
            elif self._datatype == 'float':
                if float(val) < float(self.spinbox['from']):
                    self.set(self._oldvalue)
            if self._datatype == 'integer':
                return int(self.spinbox.get())
            elif self._datatype == 'float':
                return float(self.spinbox.get())

    def selection(self, *args):
        '''Convenience method.'''
        return self.spinbox.selection(*args)

    def set(self, value=None):
        '''Convenience method.'''
        return self.tk.call(self.spinbox._w, 'set', value)

    def state(self, newstate):
        '''Convenience method.'''
        self.spinbox.configure(state=newstate)
        if self.label:
            self.label.configure(state=newstate)
        if self.rlabel:
            self.rlabel.configure(state=newstate)

    def trace(self, mode, callback):
        '''Convenience method.'''
        # this is really only useful if mode == 'w'
        self.variable.trace(mode, callback)
